@@ -2,7 +2,7 @@
  * messages.cpp : Information about an item
  ****************************************************************************
  * Copyright (C) 2006-2011 the VideoLAN team
- * $Id: 223c4798f2534ceda56741a5ec62e6a8f05228dd $
+ * $Id: 9c79c9972721a63a11a97018e2421dbc6813c9cf $
  *
  * Authors: Jean-Baptiste Kempf <jb (at) videolan.org>
  *
@@ -24,6 +24,8 @@
 # include "config.h"
 #endif
 
+#include "qt4.hpp"
+
 #include "dialogs/messages.hpp"
 
 #include <QPlainTextEdit>
@@ -143,7 +145,7 @@ MessagesDialog::~MessagesDialog()
 
 void MessagesDialog::changeVerbosity( int i_verbosity )
 {
-    vlc_atomic_set( &this->verbosity, i_verbosity );
+    atomic_store( &this->verbosity, i_verbosity );
 }
 
 void MessagesDialog::updateConfig()
@@ -337,7 +339,7 @@ void MessagesDialog::MsgCallback( void *self, int type, const vlc_log_t *item,
 {
     MessagesDialog *dialog = (MessagesDialog *)self;
     char *str;
-    int verbosity = vlc_atomic_get( &dialog->verbosity );
+    int verbosity = atomic_load( &dialog->verbosity );
 
     if( verbosity < 0 || verbosity < (type - VLC_MSG_ERR)
      || unlikely(vasprintf( &str, format, ap ) == -1) )
