@@ -7,19 +7,19 @@
  * Authors: Christopher Mueller <christopher.mueller@itec.uni-klu.ac.at>
  *          Christian Timmerer  <christian.timmerer@itec.uni-klu.ac.at>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef HTTPCONNECTION_H_
@@ -36,6 +36,10 @@
 #include <sstream>
 
 #include "http/IHTTPConnection.h"
+#include "http/Chunk.h"
+#include "Helper.h"
+
+#define PEEKBUFFER 4096
 
 namespace dash
 {
@@ -44,28 +48,26 @@ namespace dash
         class HTTPConnection : public IHTTPConnection
         {
             public:
-                HTTPConnection          ( const std::string& url, stream_t *stream );
+                HTTPConnection          (stream_t *stream);
                 virtual ~HTTPConnection ();
 
-                bool        init            ();
-                void        closeSocket     ();
-
+                virtual bool    init        (Chunk *chunk);
+                void            closeSocket ();
                 virtual int     read        (void *p_buffer, size_t len);
                 virtual int     peek        (const uint8_t **pp_peek, size_t i_peek);
 
-            private:
-                int                     httpSocket;
-                std::string             url;
-                std::string             hostname;
-                std::string             path;
-                std::string             request;
-                stream_t                *stream;
-                stream_t                *urlStream;
+            protected:
+                int         httpSocket;
+                stream_t    *stream;
+                uint8_t     *peekBuffer;
+                size_t      peekBufferLen;
+                int         contentLength;
 
-                void            parseURL        ();
-                bool            sendData        (const std::string& data);
-                bool            parseHeader     ();
-                std::string     readLine        ();
+                bool                sendData        (const std::string& data);
+                bool                parseHeader     ();
+                std::string         readLine        ();
+                virtual std::string prepareRequest  (Chunk *chunk);
+                bool                setUrlRelative  (Chunk *chunk);
         };
     }
 }
