@@ -46,10 +46,10 @@ function parse()
             local track = string.match( line, "soundcloud:tracks:(%d+)" )
             if track then
                 -- API magic
-                local client_id = "fDoItMDbsbZz8dY16ZzARCZmzgHBPotA"
+                local client_id = "2t9loNQH90kzJcsFCODdigxfp325aq4z"
                 -- app_version is not required by the API but we send it
                 -- anyway to remain unconspicuous
-                local app_version = "1480607078"
+                local app_version = "1489155300"
 
                 local api = vlc.stream( "https://api.soundcloud.com/i1/tracks/"..track.."/streams?client_id="..client_id.."&app_version="..app_version )
                 if not api then
