@@ -2,7 +2,7 @@
  * interlacing.c
  *****************************************************************************
  * Copyright (C) 2010 Laurent Aimar
- * $Id: ca71cef4ac33f75f56f2a531920aaefa31e3e6e4 $
+ * $Id: 6177a3aec919872fdd6d03454c5c66555ef04352 $
  *
  * Authors: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
@@ -38,7 +38,7 @@
  * You can use the non vout filter if and only if the video properties stay the
  * same (width/height/chroma/fps), at least for now.
  */
-static const char *deinterlace_modes[] = {
+static const char deinterlace_modes[][9]= {
     ""
     "discard",
     "blend",
@@ -50,11 +50,11 @@ static const char *deinterlace_modes[] = {
     "yadif2x",
     "phosphor",
     "ivtc",
-    NULL
 };
+
 static bool DeinterlaceIsModeValid(const char *mode)
 {
-    for (unsigned i = 0; deinterlace_modes[i]; i++) {
+    for (unsigned i = 0; i < ARRAY_SIZE(deinterlace_modes); i++) {
         if (!strcmp(deinterlace_modes[i], mode))
             return true;
     }
@@ -103,7 +103,7 @@ static void DeinterlaceRemove(vout_thread_t *vout)
     /* */
     strcpy(&start[0], &start[strlen("deinterlace")]);
     if (*start == ':')
-        strcpy(&start[0], &start[1]);
+        memmove(start, start + 1, strlen(start) /* + 1 - 1 */);
 
     var_SetString(vout, "video-filter", filter);
     free(filter);
@@ -182,11 +182,12 @@ void vout_InitInterlacingSupport(vout_thread_t *vout, bool is_interlaced)
 
     const module_config_t *optd = config_FindConfig(VLC_OBJECT(vout), "deinterlace");
     var_Change(vout, "deinterlace", VLC_VAR_CLEARCHOICES, NULL, NULL);
-    for (int i = 0; optd && i < optd->i_list; i++) {
-        val.i_int  = optd->pi_list[i];
-        text.psz_string = (char*)vlc_gettext(optd->ppsz_list_text[i]);
-        var_Change(vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text);
-    }
+    if (likely(optd != NULL))
+        for (unsigned i = 0; i < optd->list_count; i++) {
+            val.i_int = optd->list.i[i];
+            text.psz_string = vlc_gettext(optd->list_text[i]);
+            var_Change(vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text);
+        }
     var_AddCallback(vout, "deinterlace", DeinterlaceCallback, NULL);
     /* */
     var_Create(vout, "deinterlace-mode", VLC_VAR_STRING | VLC_VAR_DOINHERIT | VLC_VAR_HASCHOICE);
@@ -197,14 +198,16 @@ void vout_InitInterlacingSupport(vout_thread_t *vout, bool is_interlaced)
 
     const module_config_t *optm = config_FindConfig(VLC_OBJECT(vout), "deinterlace-mode");
     var_Change(vout, "deinterlace-mode", VLC_VAR_CLEARCHOICES, NULL, NULL);
-    for (int i = 0; optm && i < optm->i_list; i++) {
-        if (!DeinterlaceIsModeValid(optm->ppsz_list[i]))
-            continue;
+    if (likely(optm != NULL))
+        for (unsigned i = 0; i < optm->list_count; i++) {
+             if (!DeinterlaceIsModeValid(optm->list.psz[i]))
+                 continue;
 
-        val.psz_string  = optm->ppsz_list[i];
-        text.psz_string = (char*)vlc_gettext(optm->ppsz_list_text[i]);
-        var_Change(vout, "deinterlace-mode", VLC_VAR_ADDCHOICE, &val, &text);
-    }
+             val.psz_string  = optm->list.psz[i];
+             text.psz_string = vlc_gettext(optm->list_text[i]);
+             var_Change(vout, "deinterlace-mode", VLC_VAR_ADDCHOICE,
+                        &val, &text);
+         }
     var_AddCallback(vout, "deinterlace-mode", DeinterlaceCallback, NULL);
     /* */
     var_Create(vout, "deinterlace-needed", VLC_VAR_BOOL);
